function [X,map] = pbmread(filename)
%PBMREAD  Read a PBM (portable bitmap) file from disk.
%
%   [X,MAP] = PBMREAD('FILENAME') reads the file 'FILENAME' and returns
%   the indexed image X and associated colormap MAP.
%
%   I = PBMREAD('FILENAME') reads the file 'FILENAME' and returns the
%   intensity image I.
%
%   If no file extension is given with the filename, the extension
%   '.pbm' is used.
%
%   See also: PBMWRITE, PGMREAD, PGMWRITE, PPMREAD, PPMWRITE.

%   1998-01-25 16:00:44  Peter J. Acklam <jacklam@math.uio.no>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check number of input arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error( nargchk( 1, 1, nargin ) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify output arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nargsout = nargout;
if ( nargsout == 1 )
   isindexed = 0;
elseif ( nargsout == 2 )
   isindexed = 1;
else
   error( 'Wrong number of output arguments.' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check filename.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ( ~isstr( filename ) | isempty( filename ) )
   error( 'File name must be a non-empty string' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add extension to file name if necessary.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty( findstr( filename, '.' ) )
   filename = [ filename '.pbm' ];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open input file for reading.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen( filename, 'r' );
if ( fid == -1 )
   error( [ 'Can''t open file ''' filename ''' for reading.' ] );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify file type.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ magic, count ] = fscanf( fid, '%c', 2 );      % Read first two bytes.

if ( count < 2 )
   fclose( fid );
   error( 'End of file reached while reading magic number.' );
end

if ( magic == 'P1' )                            % Ascii encoded bitmap.
   isascii = 1;

elseif ( magic == 'P4' )                        % Binary encoded bitmap.
   isascii = 0;

elseif ( magic == 'P2' ) | ( magic == 'P5' )    % Graymap.
   fclose( fid );
   error( 'File is a portable graymap, use PGMREAD instead.' );

elseif ( magic == 'P3' ) | ( magic == 'P6' )    % Pixelmap.
   fclose( fid );
   error( 'File is a portable pixelmap, use PPMREAD instead.' );

else
   fclose( fid );
   error( 'File is neither a portable bitmap, graymap nor pixelmap.' );

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image size and maximum pixel value.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cols   = pnmgeti( fid );        % Number of columns.
rows   = pnmgeti( fid );        % Number of rows.
pixels = rows*cols;             % Total number of pixels in image.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isascii

   [ data, count ] = fscanf( fid, '%i', pixels );
   fclose( fid );                       % Close file.

   X = zeros( cols, rows );             % Initialize output matrix.
   X(1:count) = data;                   % Fill read data into matrix.
   X = X';

else

   % Read unsigned 8 bit integers.
   [ data, count ] = fread( fid, pixels, 'uint8' );
   fclose( fid );                         % Close file.

   if ( count*8 < pixels )
      disp( 'Warning: End of file reached too early.' );
   end

   T = zeros( ceil( cols/8 ), rows );   % Initialize temporary matrix.
   T(1:count) = data;                   % Fill data into matrix.
   T = T';

   X = zeros( rows, cols );             % Initialize output matrix.
   for i = 0:7
      bit = rem( T, 2 );
      C = floor( (cols+i)/8 );
      X(:,8-i:8:8*C-i) = bit(:,1:C);
      T = (T-bit)/2;
   end

end

if isindexed                    % Indexed graymap.
   map = [ 0 0 0 ; 1 1 1 ];     % Create two-colour colour map.
   X = 2 - X;
else
   X = 1 - X;
end
