function pbmwrite(arg1,arg2,arg3,arg4)
%PBMWRITE Write a PBM (portable bitmap) file to disk.
%
%   PBMWRITE(X,MAP,'FILENAME') writes a PBM file containing the indexed
%   image X and colormap MAP to the file 'FILENAME'.
%
%   PBMWRITE(X,MAP,'FILENAME','ENCODING') where 'ENCODING' is either
%   'ascii' (or 'plain') or 'binary' (or 'raw') lets the user specify
%   whether the PBM file should be ascii or binary encoded. Default is
%   'ascii'.
%
%   PBMWRITE(I,'FILENAME') writes a PBM file containing the black and
%   white image in the matrix I to the file 'FILENAME'.
%
%   PBMWRITE(I,'FILENAME','ENCODING') allows the user to specify the
%   encoding as described above.
%
%   The 'ascii' format is the most portable and is the only that allows
%   a MAXVALUE larger than 255, so it is also the most general. The
%   'binary' format files are, however, much smaller and many times
%   faster to read and write than the 'ascii' format.
%
%   If no file extension is given with the filename, the extension
%   '.pbm' is used.
%
%   See also: PBMREAD, PGMREAD, PGMWRITE, PPMREAD, PPMWRITE.

%   1997-09-02 14:46:51  Peter J. Acklam <jacklam@math.uio.no>

nargs = nargin;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check number of input arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error( nargchk( 2, 4, nargs ) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify input arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ( nargs == 2 )               % 2 input arguments

   %
   % PBMWRITE(I,'FILENAME')
   %
   I = arg1;
   filename = arg2;
   encoding = 'ascii';
   isindexed = 0;

elseif ( nargs == 4 )           % 4 input arguments

   %
   % PBMWRITE(X,MAP,'FILENAME','ENCODING')
   %
   X = arg1;
   map = arg2;
   filename = arg3;
   encoding = arg4;
   isindexed = 1;

else                            % 3 input arguments

   if ( isstr(arg2) & isstr(arg3) )

      %
      % PBMWRITE(I,'FILENAME','ENCODING')
      %
      I = arg1;
      filename = arg2;
      encoding = arg3;
      isindexed = 0;

   else

      %
      % PBMWRITE(X,MAP,'FILENAME')
      %
      X = arg1;
      map = arg2;
      filename = arg3;
      encoding = 'ascii';
      isindexed = 1;

   end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% These are no longer needed, so delete them to save memory.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear arg1 arg2 arg3 arg4

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check filename.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isstr(filename) | isempty(filename)
   error( 'File name must be a non-empty string' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add extension to file name if necessary.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty( findstr( filename, '.' ) )
   filename = [ filename '.pbm' ];
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check and simplify the encoding specification.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isstr( encoding )
   error( 'Encoding specification must be a string.' );
end

if any( encoding(1) == 'aApP' )         % Ascii (or plain) format.
   isascii = 1;
elseif any( encoding(1) == 'bBrR' )     % Binary (or raw) format.
   isascii = 0;
else
   error( 'Illegal encoding specification.' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open output file for writing.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen(filename,'w');
if ( fid == -1 )
   error( [ 'Can''t open file ''' filename ''' for writing.' ] );
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get dimension of image.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isindexed
   [ rows, cols ] = size( X );
else
   [ rows, cols ] = size( I );
end
pixels = rows*cols;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write file header.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isascii
   fprintf( fid, 'P1\n# Created by Matlab\n%i %i\n', cols, rows );
else
   fprintf( fid, 'P4\n# Created by Matlab\n%i %i\n', cols, rows );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write image. Colour images are first converted to grayscale.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

w = [ 0.2989360212937754 ; 0.5870430744511212 ; 0.1140209042551034 ];

if isascii
   char_count = 0;
   char_on_line = 35;
   format = [ '%i %i %i %i %i %i %i %i %i %i %i %i %i %i %i ' ...
              '%i %i %i %i %i %i %i %i %i %i %i %i %i %i %i ' ...
              '%i %i %i %i %i\n'];
else
   nbytesw = ceil( cols/8 );
   powtwo = pow2( 7:-1:0 );
end

for row = 1:rows

   if isindexed
      bit = 1 - round( map(X(row,1:cols),:)*w );
   else
      bit = 1 - round( I(row,1:cols))';
   end

   if isascii           % Write ascii format.

      if ( char_count > 0 )
         this_format = format(char_count*3+1:length(format));
         chars_to_eol = char_on_line - char_count;
         if ( length(bit) <= chars_to_eol )
            this_bit = bit;
            bit = [];
         else
            this_bit = bit(1:chars_to_eol);
            bit = bit(chars_to_eol+1:length(bit));
         end
         chars_to_write_now = length(this_bit);
         fprintf( fid, this_format, this_bit );
         char_count = char_count + chars_to_write_now;
         char_count = rem( char_count, char_on_line );
      end

      if ( char_count == 0 )
         fprintf( fid, format, bit );
         char_count = char_count + length(bit);
         char_count = rem( char_count, char_on_line );
      end

   else                 % Write binary format.

      newbit = zeros( 1, 8*nbytesw );
      newbit(1:cols) = bit;
      newbit = reshape( newbit', 8, nbytesw );
      int = powtwo*newbit;
      fwrite( fid, int, 'uint8' );

   end

end

fclose( fid );    % Close file.
