function [X,map] = pgmread(filename)
%PGMREAD  Read a PGM (portable graymap) file from disk.
%
%   [X,MAP] = PGMREAD('FILENAME') reads the file 'FILENAME' and returns
%   the indexed image X and associated colormap MAP.
%
%   I = PGMREAD('FILENAME') reads the file 'FILENAME' and returns the
%   intensity image I.
%
%   If no file extension is given with the filename, the extension
%   '.pgm' is used.
%
%   See also: PGMWRITE, PBMREAD, PBMWRITE, PPMREAD, PPMWRITE.

%   1998-01-25 16:00:35  Peter J. Acklam <jacklam@math.uio.no>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check number of input arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error( nargchk( 1, 1, nargin ) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify output arguments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nargsout = nargout;
if ( nargsout == 1 )
   isindexed = 0;
elseif ( nargsout == 2 )
   isindexed = 1;
else
   error( 'Wrong number of output arguments.' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check filename.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ( ~isstr( filename ) | isempty( filename ) )
   error( 'File name must be a non-empty string' );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add extension to file name if necessary.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty( findstr( filename, '.' ) )
   filename = [ filename '.pgm' ];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open input file for reading.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen( filename, 'r' );
if ( fid == -1 )
   error( [ 'Can''t open file ''' filename ''' for reading.' ] );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identify file type.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ magic, count ] = fscanf( fid, '%c', 2 );      % Read first two bytes.

if ( count < 2 )
   fclose( fid );
   error( 'End of file reached while reading magic number.' );
end

if ( magic == 'P1' ) | ( magic == 'P4' )        % Bitmap.

   fclose( fid );
   if isindexed
      [ X, map ] = pbmread( filename );         % Indexed bitmap.
   else
      X = pbmread( filename );                  % Intensity bitmap.
   end
   return;

elseif ( magic == 'P2' )                % Ascii encoded graymap.
   isascii = 1;

elseif ( magic == 'P5' )                % Binary encoded graymap.
   isascii = 0;

elseif ( magic == 'P3' ) | ( magic == 'P6' )    % Pixelmap.
   fclose( fid );
   error( 'File is a portable pixelmap, use PPMREAD instead.' );

else
   fclose( fid );
   error( 'File is neither a portable bitmap, graymap nor pixelmap.' );

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image size and maximum pixel value.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cols   = pnmgeti( fid );        % Number of columns.
rows   = pnmgeti( fid );        % Number of rows.
maxval = pnmgeti( fid );        % Maximum pixel value.
pixels = rows*cols;             % Total number of pixels in image.

%	CHECK THAT THE LIBERAL WHITESPACE SPECIFICATION FOLLOWED,
%	AND THE 70 CHAR LINE LIMIT FOR ASCII IS ALLOWED - OPTIONAL
%	AT THIS POINT A CHECK OF THE MAXVAL SHOULD BE MADE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read image data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%	AT THIS POINT A SWITHCH FOR 16-BIT FOMAT SHOULD BE ADDED
%	The maxval determines the visual scaling properties. If MAXVAL >255 then file is 16-bit
%	Maximum for the variable is 65535. Minimum is 1. No rescaling of the data should take place.
%	- jlj
if isascii
% ASCII form - jlj
%
%
%
   [ data, count ] = fscanf( fid, '%i', pixels );
else
% binary form- jlj
%	if maxval>255
%		'format'='uint16';
%	else
%		'format'='uint8';
%   [ data, count ] = fread( fid, pixels, 'format' );
   [ data, count ] = fread( fid, pixels, 'uint8' );
end

fclose( fid );                  % Close file.

if ( count < pixels )
   disp( 'Warning: End of file reached too early.' );
end

if isindexed                    % Indexed graymap.

   X = zeros( cols, rows );     % Initialize index matrix.
   X(1:count) = data;           % Fill data into matrix.
   X = X' + 1;                  % Map [0,maxval] to [1,maxval+1].

   map = [0:maxval]'/maxval;    % Create grayscale vector.
   map = map(:,ones(1,3));      % Convert to RGB colour map.

else                            % Intensity graymap.

   X = zeros( cols, rows );     % Initialize intensity matrix.
   X(1:count) = data;           % Fill read data into matrix.
   X = X'/maxval;               % Map values to [0,1] interval.

end
